<table class="table table-hover pd_save" style="font-size:small">
  <thead>
    <tr>
      <th align="center" valign="middle">Id</th>
      <th>Name</th>
      <th>Topic</th>
      <th>Publisher</th>
    </tr>
  </thead>
  <tbody>
  {% for key,dataDef in dataDefs %}
    <tr>
        <td>{{key}}</td>
        <td>{{dataDef["displayName"]}}</td>
        <td>{{dataDef["topic"]}}</td>
        <td>{{dataDef["publisher"]}}</td>
    </tr>
  {% endfor %}
  </tbody>
</table>