<div>
    <h2>No Mapbox Access Token</h2>
    <p>
        Before using the Mapbox renderer, you must have an access token. <a href="https://www.mapbox.com/help/create-api-access-token/" target="mapbox">Get one for free here</a>, then enter it in the Options dialog.
    </p>
</div>